/**
  *  \file planetscentral/console/router.h
  *  \brief Console: Router context
  */
#ifndef C2SERVER_PLANETSCENTRAL_CONSOLE_ROUTER_H
#define C2SERVER_PLANETSCENTRAL_CONSOLE_ROUTER_H

#include "planetscentral/console/context.h"
#include "planetscentral/dbclient/routerclient.h"

/** Context for talking with c2router. This protocol differs from the
    other protocols. In particular, it follows an
    one-request-per-connection paradigm.

    This implements the same feature-set as c2scon.pl. It does not yet
    attempt to bring the commands into a sensible form for scripted
    post-processing. Those would be
    - convert "200" router replies into a list of strings
    - convert "200" session replies into objects (?)
    - convert "201" replies into a number (session number)
    - convert other replies into errors */
class RouterContext : public Context {
 public:
    RouterContext();

    void enter();
    void leave();
    IntValue* processCommand(StringList_t& cmd);
    string_t getPrefix();

    bool checkConfig(string_t key, string_t value);

 private:

    bool isSessionCommand(StringList_t& cmd, StringList_t::size_type index);
    IntValue* processSessionCommand(string_t s, StringList_t& cmd, StringList_t::size_type start);
    IntValue* processCommand(StringList_t& cmd, StringList_t::size_type start);
    IntValue* callRouter(string_t cmd);

    RouterClient client;
    string_t session;
};

#endif
