/**
  *  \file planetscentral/dbclient/dbclient.h
  */
#ifndef C2SERVER_PLANETSCENTRAL_DBCLIENT_DBCLIENT_H
#define C2SERVER_PLANETSCENTRAL_DBCLIENT_DBCLIENT_H

#include <memory>
#include "cpluslib/string.h"
#include "int/data.h"
#include "planetscentral/dbclient/connection.h"

/** Database request. Convenience class to allow on-the-fly
    construction of IntDataSegment's with content. */
class DbRequest : public IntDataSegment {
 public:
    DbRequest& withString(const string_t& str);
    DbRequest& withString(const char* str);
    DbRequest& withInt(int32_t n);
    DbRequest& withInlineStringList(const StringList_t& sl);
    DbRequest& withInlineIntList(const IntList_t& il);
    DbRequest& withRawValue(IntValue* val);
};

/** Database client. This provides functions to submit requests to a
    database or application server.

    It uses PCC2 interpreter data types for input and output, which
    are converted to and from Redis format.

    This class provides buffering for input and output, and thus
    achieves about 20000 requests per second on my computer. */
class DbClient : public Connection {
 public:
    IntValue* call(const IntDataSegment& cmd);
    int32_t   callInt(const IntDataSegment& cmd);
    string_t  callString(const IntDataSegment& cmd);
    void      callVoid(const IntDataSegment& cmd);
};

/** Database client that probes for database readiness on connection. */
class DbReadyClient : public DbClient {
 public:
    void onConnect();
};

#endif
